@@ -12,7 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
@@ -21,32 +23,34 @@
  * @Title: GtkCellLayout
  *
  * #GtkCellLayout is an interface to be implemented by all objects which
- * want to provide a #GtkTreeViewColumn like API for packing cells,
- * setting attributes and data funcs.
+ * want to provide a #GtkTreeViewColumn-like API for packing cells, setting
+ * attributes and data funcs.
  *
- * One of the notable features provided by implementations of 
- * GtkCellLayout are attributes. Attributes let you set the properties
+ * One of the notable features provided by implementations of GtkCellLayout
+ * are <emphasis>attributes</emphasis>. Attributes let you set the properties
  * in flexible ways. They can just be set to constant values like regular
  * properties. But they can also be mapped to a column of the underlying
  * tree model with gtk_cell_layout_set_attributes(), which means that the value
- * of the attribute can change from cell to cell as they are rendered by
- * the cell renderer. Finally, it is possible to specify a function with
- * gtk_cell_layout_set_cell_data_func() that is called to determine the
- * value of the attribute for each cell that is rendered.
- *
- * # GtkCellLayouts as GtkBuildable
+ * of the attribute can change from cell to cell as they are rendered by the
+ * cell renderer. Finally, it is possible to specify a function with
+ * gtk_cell_layout_set_cell_data_func() that is called to determine the value
+ * of the attribute for each cell that is rendered.
  *
+ * <refsect2 id="GtkCellLayout-BUILDER-UI">
+ * <title>GtkCellLayouts as GtkBuildable</title>
+ * <para>
  * Implementations of GtkCellLayout which also implement the GtkBuildable
- * interface (#GtkCellView, #GtkIconView, #GtkComboBox,
+ * interface (#GtkCellView, #GtkIconView, #GtkComboBox, #GtkComboBoxEntry,
  * #GtkEntryCompletion, #GtkTreeViewColumn) accept GtkCellRenderer objects
- * as <child> elements in UI definitions. They support a custom <attributes>
- * element for their children, which can contain multiple <attribute>
- * elements. Each <attribute> element has a name attribute which specifies
- * a property of the cell renderer; the content of the element is the
- * attribute value.
+ * as &lt;child&gt; elements in UI definitions. They support a custom
+ * &lt;attributes&gt; element for their children, which can contain
+ * multiple &lt;attribute&gt; elements. Each &lt;attribute&gt; element has
+ * a name attribute which specifies a property of the cell renderer; the
+ * content of the element is the attribute value.
  *
- * This is an example of a UI definition fragment specifying attributes:
- * |[
+ * <example>
+ * <title>A UI definition fragment specifying attributes</title>
+ * <programlisting><![CDATA[
  * <object class="GtkCellView">
  *   <child>
  *     <object class="GtkCellRendererText"/>
@@ -55,17 +59,17 @@
  *     </attributes>
  *   </child>"
  * </object>
- * ]|
+ * ]]></programlisting>
+ * </example>
  *
  * Furthermore for implementations of GtkCellLayout that use a #GtkCellArea
  * to lay out cells (all GtkCellLayouts in GTK+ use a GtkCellArea)
- * [cell properties][cell-properties] can also be defined in the format by
- * specifying the custom <cell-packing> attribute which can contain multiple
- * <property> elements defined in the normal way.
- *
- * Here is a UI definition fragment specifying cell properties:
- *
- * |[
+ * <link linkend="cell-properties">cell properties</link> can also be defined
+ * in the format by specifying the custom &lt;cell-packing&gt; attribute which
+ * can contain multiple &lt;property&gt; elements defined in the normal way.
+ * <example>
+ * <title>A UI definition fragment specifying cell properties</title>
+ * <programlisting><![CDATA[
  * <object class="GtkTreeViewColumn">
  *   <child>
  *     <object class="GtkCellRendererText"/>
@@ -75,37 +79,39 @@
  *     </cell-packing>
  *   </child>"
  * </object>
- * ]|
- *
- * # Subclassing GtkCellLayout implementations
+ * ]]></programlisting>
+ * </example>
+ * </para>
+ * </refsect2>
  *
+ * <refsect2>
+ * <title>Subclassing GtkCellLayout implementations</title>
+ * <para>
  * When subclassing a widget that implements #GtkCellLayout like
  * #GtkIconView or #GtkComboBox, there are some considerations related
  * to the fact that these widgets internally use a #GtkCellArea.
  * The cell area is exposed as a construct-only property by these
  * widgets. This means that it is possible to e.g. do
- *
- * |[<!-- language="C" -->
+ * <informalexample><programlisting>
  * combo = g_object_new (GTK_TYPE_COMBO_BOX, "cell-area", my_cell_area, NULL);
- * ]|
- *
+ * </programlisting></informalexample>
  * to use a custom cell area with a combo box. But construct properties
- * are only initialized after instance init()
+ * are only initialized <emphasis>after</emphasis> instance init()
  * functions have run, which means that using functions which rely on
- * the existence of the cell area in your subclass’ init() function will
+ * the existence of the cell area in your subclass' init() function will
  * cause the default cell area to be instantiated. In this case, a provided
  * construct property value will be ignored (with a warning, to alert
  * you to the problem).
- *
- * |[<!-- language="C" -->
+ * <informalexample><programlisting>
  * static void
  * my_combo_box_init (MyComboBox *b)
  * {
  *   GtkCellRenderer *cell;
  *
  *   cell = gtk_cell_renderer_pixbuf_new ();
- *   // The following call causes the default cell area for combo boxes,
- *   // a GtkCellAreaBox, to be instantiated
+ *   /&ast; The following call causes the default cell area for combo boxes,
+ *    &ast; a GtkCellAreaBox, to be instantiated
+ *    &ast;/
  *   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (b), cell, FALSE);
  *   ...
  * }
@@ -113,16 +119,19 @@
  * GtkWidget *
  * my_combo_box_new (GtkCellArea *area)
  * {
- *   // This call is going to cause a warning about area being ignored
+ *   /&ast; This call is going to cause a warning
+ *    &ast; about area being ignored
+ *    &ast;/
  *   return g_object_new (MY_TYPE_COMBO_BOX, "cell-area", area, NULL);
  * }
- * ]|
- *
+ * </programlisting></informalexample>
  * If supporting alternative cell areas with your derived widget is
  * not important, then this does not have to concern you. If you want
  * to support alternative cell areas, you can do so by moving the
  * problematic calls out of init() and into a constructor()
  * for your class.
+ * </para>
+ * </refsect2>
  */
 
 #include "config.h"
@@ -477,7 +486,7 @@ gtk_cell_layout_set_attributes (GtkCellLayout   *cell_layout,
  * The @column is the column of the model to get a value from, and the
  * @attribute is the parameter on @cell to be set from the value. So for
  * example if column 2 of the model contains strings, you could have the
- * “text” attribute of a #GtkCellRendererText get its values from column 2.
+ * "text" attribute of a #GtkCellRendererText get its values from column 2.
  *
  * Since: 2.4
  */
@@ -500,13 +509,13 @@ gtk_cell_layout_add_attribute (GtkCellLayout   *cell_layout,
  * @cell_layout: a #GtkCellLayout
  * @cell: a #GtkCellRenderer
  * @func: (allow-none): the #GtkCellLayoutDataFunc to use, or %NULL
- * @func_data: (closure): user data for @func
+ * @func_data: user data for @func
  * @destroy: destroy notify for @func_data
  *
  * Sets the #GtkCellLayoutDataFunc to use for @cell_layout.
  *
  * This function is used instead of the standard attributes mapping
- * for setting the column value, and should set the value of @cell_layout’s
+ * for setting the column value, and should set the value of @cell_layout's
  * cell renderer(s) as appropriate.
  *
  * @func may be %NULL to remove a previously set function.
@@ -577,7 +586,7 @@ gtk_cell_layout_reorder (GtkCellLayout   *cell_layout,
  *
  * Returns the cell renderers which have been added to @cell_layout.
  *
- * Returns: (element-type GtkCellRenderer) (transfer container):
+ * Return value: (element-type GtkCellRenderer) (transfer container):
  *     a list of cell renderers. The list, but not the renderers has
  *     been newly allocated and should be freed with g_list_free()
  *     when no longer needed.
@@ -600,8 +609,7 @@ gtk_cell_layout_get_cells (GtkCellLayout *cell_layout)
  * if called on a #GtkCellArea or might be %NULL if no #GtkCellArea
  * is used by @cell_layout.
  *
- * Returns: (transfer none) (nullable): the cell area used by @cell_layout,
- * or %NULL in case no cell area is used.
+ * Return value: (transfer none): the cell area used by @cell_layout.
  *
  * Since: 3.0
  */
@@ -623,9 +631,7 @@ gtk_cell_layout_get_area (GtkCellLayout *cell_layout)
 typedef struct {
   GtkCellLayout   *cell_layout;
   GtkCellRenderer *renderer;
-  GtkBuilder      *builder;
   gchar           *attr_name;
-  GString         *string;
 } AttributesSubParserData;
 
 static void
@@ -636,90 +642,63 @@ attributes_start_element (GMarkupParseContext *context,
 			  gpointer             user_data,
 			  GError             **error)
 {
-  AttributesSubParserData *data = (AttributesSubParserData*)user_data;
+  AttributesSubParserData *parser_data = (AttributesSubParserData*)user_data;
+  guint i;
 
   if (strcmp (element_name, "attribute") == 0)
     {
-      const gchar *name;
-
-      if (!_gtk_builder_check_parent (data->builder, context, "attributes", error))
-        return;
-
-      if (!g_markup_collect_attributes (element_name, names, values, error,
-                                        G_MARKUP_COLLECT_STRING, "name", &name,
-                                        G_MARKUP_COLLECT_INVALID))
-        {
-          _gtk_builder_prefix_error (data->builder, context, error);
-          return;
-        }
-
-      data->attr_name = g_strdup (name);
+      for (i = 0; names[i]; i++)
+	if (strcmp (names[i], "name") == 0)
+	  parser_data->attr_name = g_strdup (values[i]);
     }
   else if (strcmp (element_name, "attributes") == 0)
-    {
-      if (!_gtk_builder_check_parent (data->builder, context, "child", error))
-        return;
-
-      if (!g_markup_collect_attributes (element_name, names, values, error,
-                                        G_MARKUP_COLLECT_INVALID, NULL, NULL,
-                                        G_MARKUP_COLLECT_INVALID))
-        _gtk_builder_prefix_error (data->builder, context, error);
-    }
+    return;
   else
-    {
-      _gtk_builder_error_unhandled_tag (data->builder, context,
-                                        "GtkCellLayout", element_name,
-                                        error);
-    }
+    g_warning ("Unsupported tag for GtkCellLayout: %s\n", element_name);
 }
 
 static void
-attributes_text_element (GMarkupParseContext  *context,
-			 const gchar          *text,
-			 gsize                 text_len,
-			 gpointer              user_data,
-			 GError              **error)
+attributes_text_element (GMarkupParseContext *context,
+			 const gchar         *text,
+			 gsize                text_len,
+			 gpointer             user_data,
+			 GError             **error)
 {
-  AttributesSubParserData *data = (AttributesSubParserData*)user_data;
-
-  if (data->attr_name)
-    g_string_append_len (data->string, text, text_len);
-}
-
-static void
-attributes_end_element (GMarkupParseContext  *context,
-			const gchar          *element_name,
-			gpointer              user_data,
-			GError              **error)
-{
-  AttributesSubParserData *data = (AttributesSubParserData*)user_data;
-  GValue val = G_VALUE_INIT;
-
-  if (!data->attr_name)
+  AttributesSubParserData *parser_data = (AttributesSubParserData*)user_data;
+  glong l;
+  gchar *endptr;
+  gchar *string;
+  
+  if (!parser_data->attr_name)
     return;
 
-  if (!gtk_builder_value_from_string_type (data->builder, G_TYPE_INT, data->string->str, &val, error))
+  string = g_strndup (text, text_len);
+  errno = 0;
+  l = g_ascii_strtoll (string, &endptr, 0);
+  if (errno || endptr == string)
     {
-      _gtk_builder_prefix_error (data->builder, context, error);
-       return;
+      g_set_error (error, 
+                   GTK_BUILDER_ERROR,
+                   GTK_BUILDER_ERROR_INVALID_VALUE,
+                   "Could not parse integer `%s'",
+                   string);
+      g_free (string);
+      return;
     }
+  g_free (string);
 
-  gtk_cell_layout_add_attribute (data->cell_layout,
-				 data->renderer,
-				 data->attr_name,
-                                 g_value_get_int (&val));
-
-  g_free (data->attr_name);
-  data->attr_name = NULL;
-
-  g_string_set_size (data->string, 0);
+  gtk_cell_layout_add_attribute (parser_data->cell_layout,
+				 parser_data->renderer,
+				 parser_data->attr_name, l);
+  g_free (parser_data->attr_name);
+  parser_data->attr_name = NULL;
 }
 
 static const GMarkupParser attributes_parser =
   {
     attributes_start_element,
-    attributes_end_element,
-    attributes_text_element
+    NULL,
+    attributes_text_element,
   };
 
 
@@ -732,7 +711,7 @@ gtk_cell_layout_buildable_set_cell_property (GtkCellArea     *area,
 					     const gchar     *value)
 {
   GParamSpec *pspec;
-  GValue gvalue = G_VALUE_INIT;
+  GValue gvalue = { 0, };
   GError *error = NULL;
 
   pspec = gtk_cell_area_class_find_cell_property (GTK_CELL_AREA_GET_CLASS (area), name);
@@ -777,48 +756,33 @@ cell_packing_start_element (GMarkupParseContext *context,
 			    gpointer             user_data,
 			    GError             **error)
 {
-  CellPackingSubParserData *data = (CellPackingSubParserData*)user_data;
+  CellPackingSubParserData *parser_data = (CellPackingSubParserData*)user_data;
+  guint i;
 
   if (strcmp (element_name, "property") == 0)
     {
-      const gchar *name;
-      gboolean translatable = FALSE;
-      const gchar *ctx = NULL;
-
-      if (!_gtk_builder_check_parent (data->builder, context, "cell-packing", error))
-        return;
-
-      if (!g_markup_collect_attributes (element_name, names, values, error,
-                                        G_MARKUP_COLLECT_STRING, "name", &name,
-                                        G_MARKUP_COLLECT_BOOLEAN|G_MARKUP_COLLECT_OPTIONAL, "translatable", &translatable,
-                                        G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "comments", NULL,
-                                        G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "context", &ctx,
-                                        G_MARKUP_COLLECT_INVALID))
-       {
-         _gtk_builder_prefix_error (data->builder, context, error);
-         return;
-       }
-
-      data->cell_prop_name = g_strdup (name);
-      data->translatable = translatable;
-      data->context = g_strdup (ctx);
+      for (i = 0; names[i]; i++)
+	if (strcmp (names[i], "name") == 0)
+	  parser_data->cell_prop_name = g_strdup (values[i]);
+	else if (strcmp (names[i], "translatable") == 0)
+	  {
+	    if (!_gtk_builder_boolean_from_string (values[i],
+						   &parser_data->translatable,
+						   error))
+	      return;
+	  }
+	else if (strcmp (names[i], "comments") == 0)
+	  ; /* for translators */
+	else if (strcmp (names[i], "context") == 0)
+	  parser_data->context = g_strdup (values[i]);
+	else
+	  g_warning ("Unsupported attribute for GtkCellLayout Cell "
+		     "property: %s\n", names[i]);
     }
   else if (strcmp (element_name, "cell-packing") == 0)
-    {
-      if (!_gtk_builder_check_parent (data->builder, context, "child", error))
-        return;
-
-      if (!g_markup_collect_attributes (element_name, names, values, error,
-                                        G_MARKUP_COLLECT_INVALID, NULL, NULL,
-                                        G_MARKUP_COLLECT_INVALID))
-        _gtk_builder_prefix_error (data->builder, context, error);
-    }
+    return;
   else
-    {
-      _gtk_builder_error_unhandled_tag (data->builder, context,
-                                        "GtkCellLayout", element_name,
-                                        error);
-    }
+    g_warning ("Unsupported tag for GtkCellLayout: %s\n", element_name);
 }
 
 static void
@@ -828,10 +792,10 @@ cell_packing_text_element (GMarkupParseContext *context,
 			   gpointer             user_data,
 			   GError             **error)
 {
-  CellPackingSubParserData *data = (CellPackingSubParserData*)user_data;
+  CellPackingSubParserData *parser_data = (CellPackingSubParserData*)user_data;
 
-  if (data->cell_prop_name)
-    g_string_append_len (data->string, text, text_len);
+  if (parser_data->cell_prop_name)
+    g_string_append_len (parser_data->string, text, text_len);
 }
 
 static void
@@ -840,44 +804,45 @@ cell_packing_end_element (GMarkupParseContext *context,
 			  gpointer             user_data,
 			  GError             **error)
 {
-  CellPackingSubParserData *data = (CellPackingSubParserData*)user_data;
+  CellPackingSubParserData *parser_data = (CellPackingSubParserData*)user_data;
   GtkCellArea *area;
 
-  area = gtk_cell_layout_get_area (data->cell_layout);
+  area = gtk_cell_layout_get_area (parser_data->cell_layout);
 
   if (area)
     {
       /* translate the string */
-      if (data->string->len && data->translatable)
+      if (parser_data->string->len && parser_data->translatable)
 	{
-	  const gchar *translated;
+	  gchar *translated;
 	  const gchar* domain;
 
-	  domain = gtk_builder_get_translation_domain (data->builder);
+	  domain = gtk_builder_get_translation_domain (parser_data->builder);
 
 	  translated = _gtk_builder_parser_translate (domain,
-						      data->context,
-						      data->string->str);
-	  g_string_assign (data->string, translated);
+						      parser_data->context,
+						      parser_data->string->str);
+	  g_string_set_size (parser_data->string, 0);
+	  g_string_append (parser_data->string, translated);
 	}
 
-      if (data->cell_prop_name)
-	gtk_cell_layout_buildable_set_cell_property (area,
-						     data->builder,
-						     data->renderer,
-						     data->cell_prop_name,
-						     data->string->str);
+      if (parser_data->cell_prop_name)
+	gtk_cell_layout_buildable_set_cell_property (area, 
+						     parser_data->builder,
+						     parser_data->renderer,
+						     parser_data->cell_prop_name,
+						     parser_data->string->str);
     }
   else
     g_warning ("%s does not have an internal GtkCellArea class and cannot apply child cell properties",
-	       g_type_name (G_OBJECT_TYPE (data->cell_layout)));
+	       g_type_name (G_OBJECT_TYPE (parser_data->cell_layout)));
 
-  g_string_set_size (data->string, 0);
-  g_free (data->cell_prop_name);
-  g_free (data->context);
-  data->cell_prop_name = NULL;
-  data->context = NULL;
-  data->translatable = FALSE;
+  g_string_set_size (parser_data->string, 0);
+  g_free (parser_data->cell_prop_name);
+  g_free (parser_data->context);
+  parser_data->cell_prop_name = NULL;
+  parser_data->context = NULL;
+  parser_data->translatable = FALSE;
 }
 
 
@@ -885,7 +850,7 @@ static const GMarkupParser cell_packing_parser =
   {
     cell_packing_start_element,
     cell_packing_end_element,
-    cell_packing_text_element
+    cell_packing_text_element,
   };
 
 gboolean
@@ -907,13 +872,10 @@ _gtk_cell_layout_buildable_custom_tag_start (GtkBuildable  *buildable,
       attr_data = g_slice_new0 (AttributesSubParserData);
       attr_data->cell_layout = GTK_CELL_LAYOUT (buildable);
       attr_data->renderer = GTK_CELL_RENDERER (child);
-      attr_data->builder = builder;
       attr_data->attr_name = NULL;
-      attr_data->string = g_string_new ("");
 
       *parser = attributes_parser;
       *data = attr_data;
-
       return TRUE;
     }
   else if (strcmp (tagname, "cell-packing") == 0)
@@ -926,7 +888,6 @@ _gtk_cell_layout_buildable_custom_tag_start (GtkBuildable  *buildable,
 
       *parser = cell_packing_parser;
       *data = packing_data;
-
       return TRUE;
     }
 
@@ -947,9 +908,7 @@ _gtk_cell_layout_buildable_custom_tag_end (GtkBuildable *buildable,
     {
       attr_data = (AttributesSubParserData*)data;
       g_assert (!attr_data->attr_name);
-      g_string_free (attr_data->string, TRUE);
       g_slice_free (AttributesSubParserData, attr_data);
-
       return TRUE;
     }
   else if (strcmp (tagname, "cell-packing") == 0)
@@ -957,7 +916,6 @@ _gtk_cell_layout_buildable_custom_tag_end (GtkBuildable *buildable,
       packing_data = (CellPackingSubParserData *)data;
       g_string_free (packing_data->string, TRUE);
       g_slice_free (CellPackingSubParserData, packing_data);
-
       return TRUE;
     }
   return FALSE;
